@@ -2,6 +2,7 @@
  * vi: set noexpandtab sts=8 sw=8:
  *
  * Copyright (C) 2007 OpenMoko, Inc
+ * Copyright (C) 2009 Sebastian Krzyszkowiak <seba.dos1@gmail.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -17,22 +18,18 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <gmodule.h>
 #include <glib.h>
 #include <string.h>
 #include <pk-backend.h>
-#include <pk-backend-internal.h>
-#include <pk-debug.h>
-#include <pk-package-ids.h>
-#include <pk-enum.h>
+#include <egg-debug.h>
 
+#include <stdio.h>
 #include <libopkg/opkg.h>
 
-static opkg_t *opkg;
-
 enum {
 	SEARCH_NAME,
 	SEARCH_DESCRIPTION,
@@ -43,12 +40,12 @@ enum {
 typedef struct {
 	gint search_type;
 	gchar *needle;
-	PkFilterEnum filters;
+	PkBitfield filters;
 	PkBackend *backend;
 } SearchParams;
 
 static void
-opkg_unknown_error (PkBackend *backend, gint error_code, gchar *failed_cmd)
+opkg_unknown_error (PkBackend *backend, gint error_code, const gchar *failed_cmd)
 {
 	gchar *msg;
 
@@ -64,7 +61,7 @@ opkg_unknown_error (PkBackend *backend, gint error_code, gchar *failed_cmd)
  * check an opkg package for known GUI dependancies
  */
 static gboolean
-opkg_is_gui_pkg (opkg_package_t *pkg)
+opkg_is_gui_pkg (pkg_t *pkg)
 {
 
   /* TODO: check appropriate tag */
@@ -86,7 +83,7 @@ opkg_is_gui_pkg (opkg_package_t *pkg)
  * check an opkg package to determine if it is a development package
  */
 static gboolean
-opkg_is_devel_pkg (opkg_package_t *pkg)
+opkg_is_devel_pkg (pkg_t *pkg)
 {
   if (g_strrstr (pkg->name, "-dev"))
       return TRUE;
@@ -106,8 +103,8 @@ opkg_is_devel_pkg (opkg_package_t *pkg)
  *
  * returns true if the tag is present
  */
-gboolean
-opkg_check_tag (opkg_package_t *pkg, gchar *tag)
+static gboolean 
+opkg_check_tag (pkg_t *pkg, const gchar *tag)
 {
 	if (pkg->tags && tag)
 		return (g_strrstr (pkg->tags, tag) != NULL);
@@ -120,7 +117,7 @@ handle_install_error (PkBackend *backend, int err)
 {
 	switch (err)
 	{
-	case OPKG_NO_ERROR:
+/*	case OPKG_NO_ERROR:
 		break;
 	case OPKG_PACKAGE_NOT_INSTALLED:
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_INSTALLED, NULL);
@@ -142,7 +139,7 @@ handle_install_error (PkBackend *backend, int err)
 		break;
 	case OPKG_PACKAGE_NOT_AVAILABLE:
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_FOUND, NULL);
-		break;
+		break;*/
 	default:
 		opkg_unknown_error (backend, err, "Update package");
 	}
@@ -154,7 +151,7 @@ handle_install_error (PkBackend *backend, int err)
 static void
 backend_initialize (PkBackend *backend)
 {
-	opkg = opkg_new ();
+	int opkg = opkg_new ();
 
 	if (!opkg) {
 		pk_backend_error_code (backend,
@@ -164,8 +161,8 @@ backend_initialize (PkBackend *backend)
 	}
 
 #ifdef OPKG_OFFLINE_ROOT
-	opkg_set_option (opkg, "offline_root", OPKG_OFFLINE_ROOT);
-	opkg_re_read_config_files (opkg);
+	opkg_set_option ((char *) "offline_root", OPKG_OFFLINE_ROOT);
+	opkg_re_read_config_files ();
 #endif
 
 }
@@ -176,22 +173,22 @@ backend_initialize (PkBackend *backend)
 static void
 backend_destroy (PkBackend *backend)
 {
-	opkg_free (opkg);
+	opkg_free ();
 }
 
 
 static void
-pk_opkg_progress_cb (opkg_t *opkg, const opkg_progress_data_t *pdata, void *data)
+pk_opkg_progress_cb (const opkg_progress_data_t *pdata, void *data)
 {
-	PkBackend *backend = PK_BACKEND (data);
+	PkBackend *backend = (PkBackend*) data;
 	if (!backend)
 		return;
 
 	pk_backend_set_percentage (backend, pdata->percentage);
-	if (pdata->package)
+	if (pdata->pkg)
 	{
 		gchar *uid;
-		opkg_package_t *pkg = pdata->package;
+		pkg_t *pkg = pdata->pkg;
 		gint status = PK_INFO_ENUM_UNKNOWN;
 
 		uid = g_strdup_printf ("%s;%s;%s;",
@@ -227,12 +224,12 @@ backend_refresh_cache_thread (PkBackend *backend)
 {
 	int ret;
 
-	ret = opkg_update_package_lists (opkg, pk_opkg_progress_cb, backend);
+	ret = opkg_update_package_lists (pk_opkg_progress_cb, backend);
 
 	if (ret) {
-		if (ret == OPKG_DOWNLOAD_FAILED)
-			pk_backend_error_code (backend, PK_ERROR_ENUM_REPO_NOT_AVAILABLE, NULL);
-		else
+//		if (ret == OPKG_DOWNLOAD_FAILED)
+//			pk_backend_error_code (backend, PK_ERROR_ENUM_REPO_NOT_AVAILABLE, NULL);
+//		else
 			opkg_unknown_error (backend, ret, "Refreshing cache");
 	}
 	pk_backend_finished (backend);
@@ -258,13 +255,13 @@ backend_refresh_cache (PkBackend *backend, gboolean force)
  */
 
 static void
-pk_opkg_package_list_cb (opkg_t *opkg, opkg_package_t *pkg, void *data)
+pk_opkg_package_list_cb (pkg_t *pkg, void *data)
 {
 	SearchParams *params = (SearchParams*) data;
 	gchar *uid;
 	gchar *haystack;
 	gint status, match;
-	PkFilterEnum filters = params->filters;
+	PkBitfield filters = params->filters;
 
 	if (!pkg->name)
 		return;
@@ -300,24 +297,30 @@ pk_opkg_package_list_cb (opkg_t *opkg, opkg_package_t *pkg, void *data)
 	uid = g_strdup_printf ("%s;%s;%s;",
 		pkg->name, pkg->version, pkg->architecture);
 
-	if (pkg->installed)
+	if (pkg->state_status == SS_INSTALLED)
 		status = PK_INFO_ENUM_INSTALLED;
 	else
 		status = PK_INFO_ENUM_AVAILABLE;
 
 	/* check filters */
 
-	if ((filters & PK_FILTER_ENUM_DEVELOPMENT) && !opkg_is_devel_pkg (pkg))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_DEVELOPMENT) && 
+                !opkg_is_devel_pkg (pkg))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_NOT_DEVELOPMENT) && opkg_is_devel_pkg (pkg))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_DEVELOPMENT) && 
+                opkg_is_devel_pkg (pkg))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_GUI) && !opkg_is_gui_pkg (pkg))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_GUI) && 
+                !opkg_is_gui_pkg (pkg))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_NOT_GUI) && opkg_is_gui_pkg (pkg))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_GUI) && 
+                opkg_is_gui_pkg (pkg))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_INSTALLED) && (!pkg->installed))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_INSTALLED) && 
+                (pkg->state_status != SS_INSTALLED))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_NOT_INSTALLED) && (pkg->installed))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_INSTALLED) && 
+                (pkg->state_status == SS_INSTALLED))
 		goto end_handle;
 
 	pk_backend_package (params->backend, status, uid, pkg->description);
@@ -334,18 +337,18 @@ backend_search_thread (PkBackend *backend)
 
 	params = pk_backend_get_pointer (backend, "search-params");
 
-	opkg_list_packages (opkg, pk_opkg_package_list_cb, params);
+	opkg_list_packages (pk_opkg_package_list_cb, params);
 
 	pk_backend_finished (params->backend);
 
 	g_free (params->needle);
 	g_free (params);
 
-	return FALSE;
+	return TRUE;
 }
 
 static void
-backend_search_name (PkBackend *backend, PkFilterEnum filters, const gchar *search)
+backend_search_name (PkBackend *backend, PkBitfield filters, gchar **search)
 {
 	SearchParams *params;
 
@@ -356,7 +359,7 @@ backend_search_name (PkBackend *backend, PkFilterEnum filters, const gchar *sear
 	params = g_new0 (SearchParams, 1);
 	params->filters = filters;
 	params->search_type = SEARCH_NAME;
-	params->needle = g_utf8_strdown (search, -1);
+	params->needle = g_utf8_strdown (search[0], -1);
 	params->backend = backend;
 
 	pk_backend_set_pointer (backend, "search-params", params);
@@ -367,7 +370,7 @@ backend_search_name (PkBackend *backend, PkFilterEnum filters, const gchar *sear
  * backend_search_description:
  */
 static void
-backend_search_description (PkBackend *backend, PkFilterEnum filters, const gchar *search)
+backend_search_description (PkBackend *backend, PkBitfield filters, gchar **search)
 {
 	SearchParams *params;
 
@@ -378,7 +381,7 @@ backend_search_description (PkBackend *backend, PkFilterEnum filters, const gcha
 	params = g_new0 (SearchParams, 1);
 	params->filters = filters;
 	params->search_type = SEARCH_DESCRIPTION;
-	params->needle = g_utf8_strdown (search, -1);
+	params->needle = g_utf8_strdown (search[0], -1);
 	params->backend = backend;
 
 	pk_backend_set_pointer (backend, "search-params", params);
@@ -386,7 +389,7 @@ backend_search_description (PkBackend *backend, PkFilterEnum filters, const gcha
 }
 
 static void
-backend_search_group (PkBackend *backend, PkFilterEnum filters, const gchar *search)
+backend_search_group (PkBackend *backend, PkBitfield filters, gchar **search)
 {
 	SearchParams *params;
 
@@ -397,7 +400,7 @@ backend_search_group (PkBackend *backend, PkFilterEnum filters, const gchar *sea
 	params = g_new0 (SearchParams, 1);
 	params->filters = filters;
 	params->search_type = SEARCH_TAG;
-	params->needle = g_strdup_printf ("group::%s", search);
+	params->needle = g_strdup_printf ("group::%s", search[0]);
 	params->backend = backend;
 
 	pk_backend_set_pointer (backend, "search-params", params);
@@ -408,9 +411,9 @@ backend_search_group (PkBackend *backend, PkFilterEnum filters, const gchar *sea
 static gboolean
 backend_install_packages_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	gint err, i;
 	gchar **package_ids;
+	gchar **parts;
 
 	package_ids = pk_backend_get_strv (backend, "pkids");
 
@@ -418,15 +421,15 @@ backend_install_packages_thread (PkBackend *backend)
 
 	for (i = 0; package_ids[i]; i++)
 	{
-		pk_backend_package (backend, PK_INFO_ENUM_INSTALLING, package_ids[0], NULL);
+		pk_backend_package (backend, PK_INFO_ENUM_INSTALLING, package_ids[i], NULL);
 
-		pi = pk_package_id_new_from_string (package_ids[0]);
+		parts = pk_package_id_split (package_ids[i]);
 
-		err = opkg_install_package (opkg, pi->name, pk_opkg_progress_cb, backend);
+		err = opkg_install_package (parts[PK_PACKAGE_ID_NAME], pk_opkg_progress_cb, backend);
 		if (err)
 			handle_install_error (backend, err);
 
-		pk_package_id_free (pi);
+		g_strfreev (parts);
 		if (err != 0)
 			break;
 	}
@@ -436,7 +439,7 @@ backend_install_packages_thread (PkBackend *backend)
 }
 
 static void
-backend_install_packages (PkBackend *backend, gchar **package_ids)
+backend_install_packages (PkBackend *backend, gboolean only_trusted, gchar **package_ids)
 {
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 	pk_backend_set_status (backend, PK_STATUS_ENUM_INSTALL);
@@ -449,9 +452,9 @@ backend_install_packages (PkBackend *backend, gchar **package_ids)
 static gboolean
 backend_remove_packages_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	gint err, i;
 	gchar **package_ids;
+	gchar **parts;
 	gboolean allow_deps;
 	gboolean autoremove;
 	gpointer *data;
@@ -463,29 +466,30 @@ backend_remove_packages_thread (PkBackend *backend)
 	autoremove = GPOINTER_TO_INT (data[2]);
 	g_free (data);
 
-	opkg_set_option (opkg, "autoremove", &autoremove);
-	opkg_set_option (opkg, "force_removal_of_dependent_packages", &allow_deps);
+	opkg_set_option ((char *)"autoremove", &autoremove);
+	opkg_set_option ((char *)"force_removal_of_dependent_packages", &allow_deps);
 
 	err = 0;
 
 	for (i = 0; package_ids[i]; i++)
 	{
-		pi = pk_package_id_new_from_string (package_ids[0]);
-		pk_backend_package (backend, PK_INFO_ENUM_REMOVING, package_ids[0], NULL);
+		pk_backend_package (backend, PK_INFO_ENUM_REMOVING, package_ids[i], NULL);
 
-		err = opkg_remove_package (opkg, pi->name, pk_opkg_progress_cb, backend);
+		parts = pk_package_id_split (package_ids[i]);
+
+		err = opkg_remove_package (parts[PK_PACKAGE_ID_NAME], pk_opkg_progress_cb, backend);
 
 		switch (err)
 		{
-		case OPKG_NO_ERROR:
-			break;
-		case OPKG_PACKAGE_NOT_INSTALLED:
-			pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_INSTALLED, NULL);
-			break;
+		//case OPKG_NO_ERROR:
+		//	break;
+		//case OPKG_PACKAGE_NOT_INSTALLED:
+		//	pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_INSTALLED, NULL);
+		//	break;
 		default:
 			opkg_unknown_error (backend, err, "Remove");
 		}
-		pk_package_id_free (pi);
+		g_strfreev (parts);
 
 		if (err != 0)
 			break;
@@ -519,12 +523,14 @@ backend_remove_packages (PkBackend *backend, gchar **package_ids, gboolean allow
 /**
  * backend_get_filters:
  */
-static PkFilterEnum
+static PkBitfield
 backend_get_filters (PkBackend *backend)
 {
-	return (PK_FILTER_ENUM_INSTALLED |
-		PK_FILTER_ENUM_DEVELOPMENT |
-		PK_FILTER_ENUM_GUI);
+	return pk_bitfield_from_enums (
+		PK_FILTER_ENUM_INSTALLED,
+		PK_FILTER_ENUM_DEVELOPMENT,
+		PK_FILTER_ENUM_GUI,
+		-1);
 }
 
 
@@ -533,7 +539,8 @@ backend_update_system_thread (PkBackend *backend)
 {
 	gint err;
 
-	err = opkg_upgrade_all (opkg, pk_opkg_progress_cb, backend);
+	/* FIXME: support only_trusted */
+	err = opkg_upgrade_all (pk_opkg_progress_cb, backend);
 
 	if (err)
 		opkg_unknown_error (backend, err, "Upgrading system");
@@ -543,7 +550,7 @@ backend_update_system_thread (PkBackend *backend)
 }
 
 static void
-backend_update_system (PkBackend *backend)
+backend_update_system (PkBackend *backend, gboolean only_trusted)
 {
 	pk_backend_set_status (backend, PK_STATUS_ENUM_UPDATE);
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
@@ -557,34 +564,34 @@ backend_update_system (PkBackend *backend)
 static gboolean
 backend_update_package_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
+        gchar **parts;
 	gint err = 0;
 	const gchar *package_id;
 
+	/* FIXME: support only_trusted */
 	package_id = pk_backend_get_string (backend, "pkgid");
-	pi = pk_package_id_new_from_string (package_id);
+	parts = pk_package_id_split (package_id);
 
-	if (!pi->name || !pi->version)
+	if (!parts)
 	{
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_FOUND,
 				"Package not found");
-		pk_package_id_free (pi);
 		pk_backend_finished (backend);
 		return FALSE;
 	}
 
-	err = opkg_upgrade_package (opkg, pi->name, pk_opkg_progress_cb, backend);
+	err = opkg_upgrade_package (parts[PK_PACKAGE_ID_NAME], pk_opkg_progress_cb, backend);
 	if (err)
 		handle_install_error (backend, err);
 
 
-	pk_package_id_free (pi);
+	g_strfreev (parts);
 	pk_backend_finished (backend);
 	return (err != 0);
 }
 
 static void
-backend_update_packages (PkBackend *backend, gchar **package_ids)
+backend_update_packages (PkBackend *backend, gboolean only_trusted, gchar **package_ids)
 {
 	gint i;
 
@@ -602,13 +609,13 @@ backend_update_packages (PkBackend *backend, gchar **package_ids)
  */
 
 static void
-pk_opkg_list_upgradable_cb (opkg_t *opkg, opkg_package_t *pkg, void *data)
+pk_opkg_list_upgradable_cb (pkg_t *pkg, void *data)
 {
-	PkBackend *backend = PK_BACKEND (data);
+	PkBackend *backend = (PkBackend*) data;
 	gchar *uid;
 	gint status;
 
-	if (pkg->installed)
+	if (pkg->state_status == SS_INSTALLED)
 		status = PK_INFO_ENUM_INSTALLED;
 	else
 		status = PK_INFO_ENUM_AVAILABLE;
@@ -623,13 +630,13 @@ pk_opkg_list_upgradable_cb (opkg_t *opkg, opkg_package_t *pkg, void *data)
 static gboolean
 backend_get_updates_thread (PkBackend *backend)
 {
-	opkg_list_upgradable_packages (opkg, pk_opkg_list_upgradable_cb, backend);
+	opkg_list_upgradable_packages (pk_opkg_list_upgradable_cb, backend);
 	pk_backend_finished (backend);
 	return TRUE;
 }
 
 static void
-backend_get_updates (PkBackend *backend, PkFilterEnum filters)
+backend_get_updates (PkBackend *backend, PkBitfield filters)
 {
 	pk_backend_set_status (backend, PK_STATUS_ENUM_UPDATE);
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
@@ -640,16 +647,18 @@ backend_get_updates (PkBackend *backend, PkFilterEnum filters)
 /**
  * backend_get_groups:
  */
-static PkGroupEnum
+static PkBitfield
 backend_get_groups (PkBackend *backend)
 {
-	return (PK_GROUP_ENUM_COMMUNICATION |
-		PK_GROUP_ENUM_PROGRAMMING |
-		PK_GROUP_ENUM_GAMES |
-		PK_GROUP_ENUM_OTHER |
-		PK_GROUP_ENUM_INTERNET |
-		PK_GROUP_ENUM_REPOS |
-		PK_GROUP_ENUM_MAPS);
+	return pk_bitfield_from_enums (
+		PK_GROUP_ENUM_COMMUNICATION,
+		PK_GROUP_ENUM_PROGRAMMING,
+		PK_GROUP_ENUM_GAMES,
+		PK_GROUP_ENUM_OTHER,
+		PK_GROUP_ENUM_INTERNET,
+		PK_GROUP_ENUM_REPOS,
+		PK_GROUP_ENUM_MAPS,
+		-1);
 }
 
 /**
@@ -658,16 +667,18 @@ backend_get_groups (PkBackend *backend)
 static gboolean
 backend_get_details_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
-	const gchar *package_id;
+	gchar **package_ids;
+        gchar **parts;
 	int group_index;
 	PkGroupEnum group = 0;
-	opkg_package_t *pkg;
+	pkg_t *pkg;
 	gchar *newid;
 
-	package_id = pk_backend_get_string (backend, "package_id");
-	pi = pk_package_id_new_from_string (package_id);
-	if (pi == NULL)
+        package_ids = pk_backend_get_strv(backend, "package_ids");
+	parts = pk_package_id_split (package_ids[0]);
+
+
+	if (!parts)
 	{
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "invalid package id");
 		pk_backend_finished (backend);
@@ -675,8 +686,8 @@ backend_get_details_thread (PkBackend *backend)
 	}
 
 
-	pkg = opkg_find_package (opkg, pi->name, pi->version, pi->arch, pi->data);
-	pk_package_id_free (pi);
+	pkg = opkg_find_package (parts[PK_PACKAGE_ID_NAME], parts[PK_PACKAGE_ID_VERSION], parts[PK_PACKAGE_ID_ARCH], parts[PK_PACKAGE_ID_DATA]);
+	g_strfreev (parts);
 
 	if (!pkg)
 	{
@@ -685,26 +696,25 @@ backend_get_details_thread (PkBackend *backend)
 		return FALSE;
 	}
 
-	newid = g_strdup_printf ("%s;%s;%s;%s", pkg->name, pkg->version, pkg->architecture, pkg->repository);
+	newid = g_strdup_printf ("%s;%s;%s;%s", pkg->name, pkg->version, pkg->architecture, pkg->src->name);
 
 	if (pkg->tags) {
-		for (group_index = 0; group < PK_GROUP_ENUM_UNKNOWN; group_index++) {
+		for (group_index = 0; group < PK_GROUP_ENUM_LAST; group_index++) {
 			group = 1 << group_index;
 			if (!(group & backend_get_groups(backend))) continue;
-			if (opkg_check_tag(pkg, (gchar *)pk_group_enum_to_text(group))) 
+			if (opkg_check_tag(pkg, (const gchar *)pk_group_enum_to_string(group))) 
 				break;
 		}
 	}
 
-	pk_backend_details (backend, newid, NULL, group, pkg->description, pkg->url, pkg->size);
+	pk_backend_details (backend, newid, NULL, group, pkg->description, NULL, pkg->size);
 	g_free (newid);
-	opkg_package_free(pkg);
 	pk_backend_finished (backend);
 	return TRUE;
 }
 
 static void
-backend_get_details (PkBackend *backend, const gchar *package_id)
+backend_get_details (PkBackend *backend, gchar **package_ids)
 {
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 	pk_backend_thread_create (backend, backend_get_details_thread);
@@ -717,9 +727,14 @@ PK_BACKEND_OPTIONS (
 	backend_destroy,			/* destroy */
 	backend_get_groups,			/* get_groups */
 	backend_get_filters,			/* get_filters */
+	NULL,					/* get_roles */
+	NULL,					/* get_mime_types */
 	NULL,					/* cancel */
+	NULL,					/* download_packages */
+	NULL,					/* get_categories */
 	NULL,					/* get_depends */
 	backend_get_details,			/* get_details */
+	NULL,					/* get_distro_upgrades */
 	NULL,					/* get_files */
 	NULL,					/* get_packages */
 	NULL,					/* get_repo_list */
@@ -739,9 +754,12 @@ PK_BACKEND_OPTIONS (
 	NULL,					/* search_file */
 	backend_search_group,			/* search_group */
 	backend_search_name,			/* search_name */
-	NULL,					/* service_pack */
 	backend_update_packages,		/* update_packages */
 	backend_update_system,			/* update_system */
-	NULL					/* what_provides */
+	NULL,					/* what_provides */
+	NULL,					/* simulate_install_files */
+	NULL,					/* simulate_install_packages */
+	NULL,					/* simulate_remove_packages */
+	NULL					/* simulate_update_packages */
 );
 
